// Copyright(c) 1999 arton

#ifndef FILTER_CLASS_HEADER
#define FILTER_CLASS_HEADER

// This class is not COM, but C++.

LPBYTE Pack(LPVOID, LPCOLESTR, int);
HRESULT Unpack(VARIANT& v, BSTR* pResult);
LPOLESTR Unpack(LPOLESTR, LPVOID, int);

class CFilter {
public:
	enum ETOpt {
		NO_OPT = 0,
		IP_ONLY = 1,
		IP_ARP = 2,
		IP_TCP = 3,
		IP_UDP = 4,
	};
	virtual bool DataCompare(int, LPBYTE pb) const = 0;
	virtual bool CheckRange (int sz) const { return true; }
	virtual ~CFilter() {}

	static CFilter* CreateFilter(IFilterParam* pParam);
};

class CFilterNode : public CFilter
{
public:
	CFilterNode() : m_pLeftClause(NULL), m_pRightClause(NULL) {}
	CFilterNode(IFilterParam* pLeft, IFilterParam* pRight)
	{
		m_pLeftClause = CreateFilter(pLeft);
		m_pRightClause = CreateFilter(pRight);
	}
	~CFilterNode() 
	{
		delete m_pLeftClause;
		delete m_pRightClause;
	}
protected:
	CFilter* m_pLeftClause;
	CFilter* m_pRightClause;
};

class CFilterNodeAnd : public CFilterNode {
public:
	CFilterNodeAnd(IFilterParam* pLeft, IFilterParam* pRight)
		: CFilterNode(pLeft, pRight)
	{
	}
	bool DataCompare(int sz, LPBYTE pb) const
	{
		return (m_pLeftClause->DataCompare(sz, pb) && m_pRightClause->DataCompare(sz, pb));
	}
	bool CheckRange (int sz) const 
	{
		return (m_pLeftClause->CheckRange(sz) && m_pRightClause->CheckRange(sz));
	}

};

class CFilterNodeOr : public CFilterNode {
public:
	CFilterNodeOr(IFilterParam* pLeft, IFilterParam* pRight)
		: CFilterNode(pLeft, pRight)
	{
	}
	bool DataCompare(int sz, LPBYTE pb) const
	{
		return ((m_pLeftClause->CheckRange(sz) && m_pLeftClause->DataCompare(sz, pb)) 
			|| (m_pRightClause->CheckRange(sz) && m_pRightClause->DataCompare(sz, pb)));
	}
	CFilterNodeOr(CFilter* pLeft, CFilter* pRight)
	{
		m_pLeftClause = pLeft;
		m_pRightClause = pRight;
	}
};

typedef std::list<CFilter*> FilterList;
typedef std::list<CFilter*>::iterator FilterListIter;
typedef std::list<CFilter*>::const_iterator FilterListIterC;

class CFilterNodeMany : public CFilter
{
public:
	CFilterNodeMany(SAFEARRAY* pAry)
	{
		SafeArrayLock(pAry);
		IFilterParam* pParam;
		LPDISPATCH* pDisp = reinterpret_cast<LPDISPATCH*>(pAry->pvData);
		for (int i = 0; i < pAry->rgsabound[0].cElements; i++, pDisp++)
		{
			if ((*pDisp)->QueryInterface(IID_IFilterParam, (void**)&pParam) == S_OK)
			{
				CFilter* p = CreateFilter(pParam);
				if (p)
					m_list.push_back(p);
				pParam->Release();
			}
		}
		SafeArrayUnlock(pAry);
	}
	~CFilterNodeMany() 
	{
		for (FilterListIter it = m_list.begin(); it != m_list.end(); it++)
		{
			delete *it;
		}
	}
protected:
	FilterList m_list;
};

class CFilterNodeManyAnd : public CFilterNodeMany {
public:
	CFilterNodeManyAnd(SAFEARRAY* pAry)
		: CFilterNodeMany(pAry)
	{
	}
	bool DataCompare(int sz, LPBYTE pb) const
	{
		for (FilterListIterC it = m_list.begin(); it != m_list.end(); it++)
		{
			if ((*it)->DataCompare(sz, pb) == false) return false;
		}
		return true;
	}
	bool CheckRange (int sz) const 
	{
		for (FilterListIterC it = m_list.begin(); it != m_list.end(); it++)
		{
			if ((*it)->CheckRange(sz) == false) return false;
		}
		return true;
	}

};

class CFilterNodeManyOr : public CFilterNodeMany {
public:
	CFilterNodeManyOr(SAFEARRAY* pAry)
		: CFilterNodeMany(pAry)
	{
	}
	bool DataCompare(int sz, LPBYTE pb) const
	{
		for (FilterListIterC it = m_list.begin(); it != m_list.end(); it++)
		{
			if ((*it)->CheckRange(sz) && (*it)->DataCompare(sz, pb)) return true;
		}
		return false;
	}
};

class CFilterNot : public CFilter {
public:
	CFilterNot(IFilterParam* pParam)
		: m_pClause(CreateFilter(pParam)) 
	{
	}
	~CFilterNot() { delete m_pClause; }
	bool DataCompare(int sz, LPBYTE pb) const
	{
		return (m_pClause->CheckRange(sz) == false || m_pClause->DataCompare(sz, pb) == false);
	}
private:
	CFilter* m_pClause;
};

class CFilterValue : public CFilter {
public:
	CFilterValue(LPBYTE pb, int cb, int off, ETOpt e = NO_OPT)
		: m_hVal(0), m_pData(NULL), m_opt(e)
	{
		m_cb = cb;
		m_offset = off;
		if (cb <= sizeof(__int64))
		{
			CopyMemory(&m_hVal, pb, cb);
		}
		else
		{
			m_pData = new BYTE[cb];
			CopyMemory(m_pData, pb, cb);
		}
	}
	~CFilterValue()
	{
		delete[] m_pData;
	}
	inline int GetSize() const { return m_cb; }
	bool DataCompare(int, LPBYTE pb) const;
	bool CheckRange(int sz) const 
	{
		return ((m_offset + m_cb) > sz) ? false : true;
	}
private:
	LPBYTE m_pData;
	int m_cb;
	int m_offset;
	__int64 m_hVal;
	ETOpt m_opt;
};

typedef std::vector<CFilter*> AFilter;
typedef std::vector<CFilter*>::iterator AFilterIter;

#endif
