/*++

Copyright (c) 1990-2000  Microsoft Corporation

Module Name:

    open.c

Abstract:


Author:


Environment:

    Kernel mode only.

Notes:


Revision History:

    Updated for Windows 2000 - Eliyas Yakub June, 1999


--*/

#include "ntddk.h"
#include "ndis.h"
#include "Ncap.h"
#include "packet.h"

static UINT            Medium;
static NDIS_MEDIUM     MediumArray=NdisMedium802_3;


NTSTATUS
PacketOpen(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    )

/*++

Routine Description:

    This is the dispatch routine for create/open and close requests.
    These requests complete successfully.

Arguments:

    DeviceObject - Pointer to the device object.

    Irp - Pointer to the request packet.

Return Value:

    Status is returned.

--*/

{
    int i;
    LONG	numOpen;
    POPEN_INSTANCE      open;
    NTSTATUS            status = STATUS_SUCCESS;
    NDIS_STATUS     ErrorStatus;

    DebugPrint(("OpenAdapter\n"));

    if(DeviceObject == Globals.ControlDeviceObject) {
	return finishIrp(Irp, STATUS_SUCCESS, 0);
    }
    
    open = DeviceObject->DeviceExtension;

    numOpen = InterlockedIncrement(&open->OpenCount);
    if (numOpen > 1)
    {
	return finishIrp(Irp, STATUS_SUCCESS, 0);	
    }
    
    open->IrpCount = 0;

    DebugPrint(("AdapterName :%ws\n", open->AdapterName.Buffer));

    //
    //  Save the Irp here for the completeion routine to retrieve
    //
    open->OpenCloseIrp=Irp;

    //
    //  Allocate a packet pool for our xmit and receive packets
    //
    NdisAllocatePacketPool(
        &status,
        &open->PacketPool,
        TRANSMIT_PACKETS,
        sizeof(CapReserved));

    if (status != NDIS_STATUS_SUCCESS) {
        return finishIrp(Irp, STATUS_INSUFFICIENT_RESOURCES, 0);
    }

    //
    //  list to hold irp's want to reset the adapter
    //
    InitializeListHead(&open->ResetIrpList);


    //
    //  Initialize list for holding pending read requests
    //
    KeInitializeSpinLock(&open->RcvQSpinLock);
    InitializeListHead(&open->RcvList);

    //
    //  Initialize the request list
    //
    NdisAllocateSpinLock(&open->RequestSpinLock);
    InitializeListHead(&open->RequestList);


    //
    //  link up the request stored in our open block
    //
    for (i=0;i<MAX_REQUESTS;i++) {
        NdisInterlockedInsertTailList(
            &open->RequestList,
            &open->Requests[i].ListElement,
            &open->RequestSpinLock);
    }


    IoMarkIrpPending(Irp);
    Irp->IoStatus.Status = STATUS_PENDING;

    //
    //  Try to open the MAC
    //
    NdisOpenAdapter(
        &status,
        &ErrorStatus,
        &open->AdapterHandle,
        &Medium,
        &MediumArray,
        1,
	open->NdisProtocolHandle,
        open,
        &open->AdapterName,
        0,
        NULL);


    if (status != NDIS_STATUS_PENDING) {

        PacketOpenAdapterComplete(
            open,
            status,
            NDIS_STATUS_SUCCESS
            );
    }

    return(STATUS_PENDING);
}



NTSTATUS
PacketClose(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    )

/*++

Routine Description:

    This is the dispatch routine for create/open and close requests.
    These requests complete successfully.

Arguments:

    DeviceObject - Pointer to the device object.

    Irp - Pointer to the request packet.

Return Value:

    Status is returned.

--*/

{
  
    POPEN_INSTANCE      open;
    NTSTATUS            status = STATUS_SUCCESS;
    LONG                numOpen;

    DebugPrint(("CloseAdapter \n"));

    if(DeviceObject == Globals.ControlDeviceObject) {
	return finishIrp(Irp, STATUS_SUCCESS, 0);
    }

    open = DeviceObject->DeviceExtension;
    numOpen = InterlockedDecrement(&open->OpenCount);
    if (numOpen > 0)
    {
	return finishIrp(Irp, STATUS_SUCCESS, 0);	
    }

    //
    //  Save the IRP
    //
    open->OpenCloseIrp = Irp;

    IoMarkIrpPending(Irp);
    Irp->IoStatus.Status = STATUS_PENDING;

    //
    //  close the adapter
    //
    NdisCloseAdapter(
        &status,
        open->AdapterHandle
        );

    if (status != NDIS_STATUS_PENDING) {

        PacketCloseAdapterComplete(
            open,
            status
            );


    }

    return(STATUS_PENDING);
}

NTSTATUS
PacketCleanup(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    )

/*++

Routine Description:

    This is the dispatch routine for cleanup requests.
    This routine is called whenever a handle to the device
    is closed.
    
Arguments:

    DeviceObject - Pointer to the device object.

    Irp - Pointer to the request packet.

Return Value:

    Status is returned.

--*/

{

    POPEN_INSTANCE      open = DeviceObject->DeviceExtension;
    NTSTATUS            status = STATUS_SUCCESS;
    PLIST_ENTRY listEntryP;

    DebugPrint(("Packet: Cleanup\n"));

    if(DeviceObject == Globals.ControlDeviceObject) {
	return finishIrp(Irp, STATUS_SUCCESS, 0);
    }

    IoMarkIrpPending(Irp);
    Irp->IoStatus.Status = STATUS_PENDING;
    
    //
    // Cancel all the pending reads.
    // 
    
    PacketCancelReadIrps(DeviceObject);

    return finishIrp(Irp, STATUS_SUCCESS, 0);
}

VOID
PacketResetComplete(
    IN NDIS_HANDLE  ProtocolBindingContext,
    IN NDIS_STATUS  Status
    )
/*++

Routine Description:

    This routine  is called when a protocol-initiated reset operation, 
    begun with a call to NdisReset that returned NDIS_STATUS_PENDING, 
    is completed. 

Arguments:


Return Value:


--*/


{
    POPEN_INSTANCE      open;
    PIRP                irp;

    PLIST_ENTRY         resetListEntry;

    DebugPrint(("PacketResetComplte\n"));

    open= (POPEN_INSTANCE)ProtocolBindingContext;


    //
    //  remove the reset IRP from the list
    //
    resetListEntry=ExInterlockedRemoveHeadList(
                       &open->ResetIrpList,
                       &open->ResetQueueLock
                       );

    irp=CONTAINING_RECORD(resetListEntry,IRP,Tail.Overlay.ListEntry);

    if(Status == NDIS_STATUS_SUCCESS) {
        irp->IoStatus.Status = STATUS_SUCCESS;
    } else {
        irp->IoStatus.Status = STATUS_UNSUCCESSFUL;
    }

    irp->IoStatus.Information = 0;    
    IoCompleteRequest(irp, IO_NO_INCREMENT);
//    IoDecrement(open);

    DebugPrint(("PacketResetComplte exit\n"));

    return;

}



